@@ -31,6 +31,10 @@
 #include <string.h>
 #include <glib/gi18n.h>
 
+#include <libpanel-util/panel-error.h>
+#include <libpanel-util/panel-launch.h>
+#include <libpanel-util/panel-show.h>
+
 #include "panel-util.h"
 #include "panel-background.h"
 #include "panel-action-button.h"
@@ -44,7 +48,7 @@
 #include "panel-typebuiltins.h"
 #include "panel-icon-names.h"
 
-G_DEFINE_TYPE (PanelMenuBar, panel_menu_bar, GTK_TYPE_MENU_BAR);
+G_DEFINE_TYPE (PanelMenuBar, panel_menu_bar, GTK_TYPE_MENU_BAR)
 
 #define PANEL_MENU_BAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_MENU_BAR, PanelMenuBarPrivate))
 
@@ -67,60 +71,30 @@ enum {
 
 static void panel_menu_bar_update_text_gravity (PanelMenuBar *menubar);
 
-static gboolean
-panel_menu_bar_enter_applications (GtkWidget        *widget,
-				   GdkEventCrossing *event,
-				   PanelMenuBar     *menubar)
-{
-	gtk_tooltips_set_tip (panel_tooltips,
-			      GTK_WIDGET (menubar),
-			      _("Browse and run installed applications"),
-			      NULL);
-	return FALSE;
-}
-
-static gboolean
-panel_menu_bar_enter_places (GtkWidget        *widget,
-			     GdkEventCrossing *event,
-			     PanelMenuBar     *menubar)
-{
-	gtk_tooltips_set_tip (panel_tooltips,
-			      GTK_WIDGET (menubar),
-			      _("Access documents, folders and network places"),
-			      NULL);
-	return FALSE;
-}
-
-static gboolean
-panel_menu_bar_enter_desktop (GtkWidget        *widget,
-			      GdkEventCrossing *event,
-			      PanelMenuBar     *menubar)
-{
-	gtk_tooltips_set_tip (panel_tooltips,
-			      GTK_WIDGET (menubar),
-			      _("Change desktop appearance and behavior, get help, or log out"),
-			      NULL);
-	return FALSE;
-}
-
 static gboolean
 panel_menu_bar_reinit_tooltip (GtkWidget    *widget,
 			       PanelMenuBar *menubar)
 {
-	gtk_tooltips_set_tip (panel_tooltips,
-			      GTK_WIDGET (menubar),
-			      "", NULL);
+	g_object_set (menubar->priv->applications_item,
+		      "has-tooltip", TRUE, NULL);
+	g_object_set (menubar->priv->places_item,
+		      "has-tooltip", TRUE, NULL);
+	g_object_set (menubar->priv->desktop_item,
+		      "has-tooltip", TRUE, NULL);
 
 	return FALSE;
 }
 
 static gboolean
-panel_menu_bar_hide_tooltip (GtkWidget    *widget,
-			     PanelMenuBar *menubar)
+panel_menu_bar_hide_tooltip_and_focus (GtkWidget    *widget,
+				       PanelMenuBar *menubar)
 {
-	gtk_tooltips_set_tip (panel_tooltips,
-			      GTK_WIDGET (menubar),
-			      NULL, NULL);
+	/* remove focus that would be drawn on the currently focused child of
+	 * the toplevel. See bug#308632. */
+	gtk_window_set_focus (GTK_WINDOW (menubar->priv->panel->toplevel),
+			      NULL);
+
+	g_object_set (widget, "has-tooltip", FALSE, NULL);
 
 	return FALSE;
 }
@@ -128,33 +102,26 @@ panel_menu_bar_hide_tooltip (GtkWidget    *widget,
 static void
 panel_menu_bar_setup_tooltip (PanelMenuBar *menubar)
 {
-	/* Update tooltip when we enter the menu items */
-	g_signal_connect (menubar->priv->applications_item,
-			  "enter-notify-event",
-			  G_CALLBACK (panel_menu_bar_enter_applications),
-			  menubar);
-	g_signal_connect (menubar->priv->places_item,
-			  "enter-notify-event",
-			  G_CALLBACK (panel_menu_bar_enter_places),
-			  menubar);
-	g_signal_connect (menubar->priv->desktop_item,
-			  "enter-notify-event",
-			  G_CALLBACK (panel_menu_bar_enter_desktop),
-			  menubar);
+	panel_util_set_tooltip_text (menubar->priv->applications_item,
+				     _("Browse and run installed applications"));
+	panel_util_set_tooltip_text (menubar->priv->places_item,
+				     _("Access documents, folders and network places"));
+	panel_util_set_tooltip_text (menubar->priv->desktop_item,
+				     _("Change desktop appearance and behavior, get help, or log out"));
 
 	//FIXME: this doesn't handle the right-click case. Sigh.
 	/* Hide tooltip if a menu is activated */
 	g_signal_connect (menubar->priv->applications_item,
 			  "activate",
-			  G_CALLBACK (panel_menu_bar_hide_tooltip),
+			  G_CALLBACK (panel_menu_bar_hide_tooltip_and_focus),
 			  menubar);
 	g_signal_connect (menubar->priv->places_item,
 			  "activate",
-			  G_CALLBACK (panel_menu_bar_hide_tooltip),
+			  G_CALLBACK (panel_menu_bar_hide_tooltip_and_focus),
 			  menubar);
 	g_signal_connect (menubar->priv->desktop_item,
 			  "activate",
-			  G_CALLBACK (panel_menu_bar_hide_tooltip),
+			  G_CALLBACK (panel_menu_bar_hide_tooltip_and_focus),
 			  menubar);
 
 	/* Reset tooltip when the menu bar is not used */
@@ -162,13 +129,6 @@ panel_menu_bar_setup_tooltip (PanelMenuBar *menubar)
 			  "deactivate",
 			  G_CALLBACK (panel_menu_bar_reinit_tooltip),
 			  menubar);
-
-	/* Preset a tooltip that will change when the cursor enters an item
-	 * If we don't do this, the tooltip will not work on the first enter
-	 * event. */
-	gtk_tooltips_set_tip (panel_tooltips,
-			      GTK_WIDGET (menubar),
-			      "", NULL);
 }
 
 static void
@@ -180,9 +140,11 @@ panel_menu_bar_init (PanelMenuBar *menubar)
 
 	menubar->priv->info = NULL;
 
-	menubar->priv->applications_menu = create_applications_menu ("applications.menu", NULL);
+	menubar->priv->applications_menu = create_applications_menu ("applications.menu", NULL, TRUE);
 
-	menubar->priv->applications_item = gtk_image_menu_item_new_with_label (_("Applications"));
+	menubar->priv->applications_item = panel_image_menu_item_new ();
+	gtk_menu_item_set_label (GTK_MENU_ITEM (menubar->priv->applications_item),
+				 _("Applications"));
 	image = gtk_image_new_from_icon_name (PANEL_ICON_MAIN_MENU,
 					      panel_menu_bar_icon_get_size ());
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menubar->priv->applications_item),
@@ -436,33 +398,20 @@ panel_menu_bar_invoke_menu (PanelMenuBar *menubar,
 	screen = gtk_widget_get_screen (GTK_WIDGET (menubar));
 
 	if (!strcmp (callback_name, "help")) {
-		panel_show_help (screen, "user-guide.xml", "menubar");
+		panel_show_help (screen, "user-guide", "menubar", NULL);
 
 	} else if (!strcmp (callback_name, "edit")) {
 		GError *error = NULL;
 
-		panel_launch_desktop_file ("alacarte.desktop",
-					   "alacarte",
-					   screen,
-					   &error);
+		panel_launch_desktop_file_with_fallback ("alacarte.desktop",
+							 "alacarte",
+							 screen, &error);
 		if (error) {
 			g_error_free (error);
-			error = NULL;
-			panel_launch_desktop_file ("gmenu-simple-editor.desktop",
-						   "gmenu-simple-editor",
-						   screen,
-						   &error);
-		}
-		if (error) {
-			char *primary;
-			primary = g_strdup_printf (_("Could not execute '%s'"),
-						   "gmenu-simple-editor");
-                        panel_error_dialog (NULL, screen,
-                                            "cannot_exec_gmenu-simple-editor",
-					    TRUE,
-					    primary, error->message);
-			g_free (primary);
-			g_error_free (error);
+			panel_launch_desktop_file_with_fallback (
+						"gmenu-simple-editor.desktop",
+						"gmenu-simple-editor",
+						screen, NULL);
 		}
 	}
 }
