@@ -32,6 +32,7 @@
 #include <libpanel-util/panel-error.h>
 #include <libpanel-util/panel-glib.h>
 #include <libpanel-util/panel-keyfile.h>
+#include <libpanel-util/panel-xdg.h>
 
 #include "applet.h"
 #include "nothing.h"
@@ -240,24 +241,6 @@ panel_uri_exists (const char *uri)
 	return ret;
 }
 
-char *
-panel_util_icon_remove_extension (const char *icon)
-{
-	char *icon_no_extension;
-	char *p;
-
-	icon_no_extension = g_strdup (icon);
-	p = strrchr (icon_no_extension, '.');
-	if (p &&
-	    (strcmp (p, ".png") == 0 ||
-	     strcmp (p, ".xpm") == 0 ||
-	     strcmp (p, ".svg") == 0)) {
-	    *p = 0;
-	}
-
-	return icon_no_extension;
-}
-
 char *
 panel_find_icon (GtkIconTheme  *icon_theme,
 		 const char    *icon_name,
@@ -287,7 +270,7 @@ panel_find_icon (GtkIconTheme  *icon_theme,
 
 	/* This is needed because some .desktop files have an icon name *and*
 	 * an extension as icon */
-	icon_no_extension = panel_util_icon_remove_extension (icon_name);
+	icon_no_extension = panel_xdg_icon_remove_extension (icon_name);
 
 	info = gtk_icon_theme_lookup_icon (icon_theme, icon_no_extension,
 					   size, 0);
@@ -951,6 +934,10 @@ panel_util_get_icon_for_uri_known_folders (const char *uri)
 
 	path = g_filename_from_uri (uri, NULL, NULL);
 
+	/* Sanity check. We may be given a broken URI. */
+	if (path == NULL)
+		return NULL;
+
 	len = strlen (path);
 	if (path[len] == '/')
 		path[len] = '\0';
